/**
 * Place global styles in this file.
 */

body
  // Override the default font from Bootstrap
  font-family "Open Sans", sans-serif !important
  padding 0
  word-wrap break-word

i
  -webkit-font-smoothing antialiased

a
  cursor pointer
  text-decoration none !important

.g-clear-both
  clear both

.g-clear-right
  clear right

ul.dropdown-menu
  >li.dropdown-header
    padding 3px 20px 3px 8px
    margin-bottom 3px
    background-color #555
    font-weight bold
    color white
  >li>a
    padding 3px 20px 3px 8px

    &:hover
      background-color #eee
    >i
      margin-right 3px

.panel-heading
  padding 10px

.panel-title
  font-size 15px

.g-body-title
  font-size 20px
  font-weight bold
  color #555

.g-body-subtitle
  font-size 15px
  color #888
  margin-bottom 5px

.g-dialog-subtitle
  margin-top 5px
  width 100%

#g-confirm-text
  max-width 50%

.g-info-message-container
  background-color #fffee1
  color #777
  padding 6px 8px
  border 1px solid #e5e2c8

// Augment the bootstrap tabs styling a bit
.nav-tabs>li:not(.active)>a
  border 1px solid #e2e2e2

// Personal preference for less white space at bottom of dialogs
.modal-footer
  margin-top 5px

.modal-content
  border-radius 0
  max-height calc(100vh - 60px)
  overflow-y auto
  margin 30px auto

.modal-backdrop.in
  opacity 0.35

// By default bootstrap only applies form-inline rules at >=768px width
.form-inline .form-group
  display inline-block
  margin-bottom 0
  vertical-align middle

blockquote p
  font-size inherit

code
  font-family monospace

// Don't show the vertical scroll bar when we show a dialog unless we must
.modal
  overflow-y auto

.g-monospace-viewer
  font-family monospace
  border 1px solid #ccc
  background-color #f5f5f5
  border-radius 0
  overflow-x scroll
  padding 7px 11px
  white-space pre
  word-wrap initial
  font-size 13px

textarea
  max-width 100%

// Stuff that applies across the top level object lists
.g-top-level-list-title-container
  display inline-block
  overflow hidden
  white-space nowrap
  width calc(100% - 350px)
  position relative

  .g-text-fade
    height 30px
    width 45px
    position absolute
    right 0
    background linear-gradient(to right, rgba(250, 250, 252, 0), rgba(250, 250, 252, 1))

    &[public="false"]
      background linear-gradient(to right, rgba(255, 249, 234, 0), rgba(255, 249, 234, 1))

.g-list-public-status-icon
  display inline-block
  line-height 40px
  vertical-align top
  color #888
  margin-right 10px
  border-right 1px solid #dedede
  padding-right 5px
  font-size 18px
  margin-left -5px

.g-top-level-list-entry
  padding 11px 14px
  margin 2px 0
  border-radius 3px
  background-color #fafafc
  border 1px solid #eaeaea
  box-shadow inset 0 0 2px 2px #f4f4f4

  &[public="false"]
    box-shadow inset 0 0 2px 2px #fff0e0
    background-color #fff9ea
